############################################################
### Pointwise and simultaneous confidence bounds for the ###
### distribution function F                              ###
### Lutz Duembgen, Ehsan Zamanzadeh, July 2018           ###
############################################################

# The underlying data are two vectors X and R of length n,
# where (X[i],R[i]), 1 <= i <= n, are assumed to be
# independent with R[i] in {1,...,k} and
#   P(X[i] <= x | R[i] = r) = pbeta(F(x),r,k+1-r) .
# Here F is the underlying continuous distribution
# function to be estimated.
# 
# Inference on F is based on the conditional distribution
# of X, given R.


############################################################
### Pointwise confidence bounds                          ###
############################################################

ConfBandPW_Data <- function(X,R,k,alpha=0.05,
	xlim=c(1.1*min(X)-0.1*max(X), 1.1*max(X)-0.1*min(X)),
	display=FALSE,display.all=FALSE)
# For a ranked set sample with observation vectors X and R
# as above, this procedure computes (and displays)
# pointwise exact (1 - alpha)-confidence bounds for F(x).
# Precisely, the function returns a table with the
# following columns:
#    y : a vector with the order statistics of X,
#        y[1] = -Inf  and
#        y[i+1] = X_(i)  for 1 <= i <= n.
#        (Below we also refer to y[n+2] = Inf.)
#    a1 : For 1 <= i <= n+1 and y[i] < x <= y[i+1],
#        a1[i] is a lower (1 - alpha)-confidence bound
#        for F(x).
#    b1 : For 1 <= i <= n and y[i] <= x < y[i+1],
#        b1[i] is an upper (1 - alpha)-confidence bound
#        for F(x).
#    a2 : For 1 <= i <= n+1 and y[i] < x <= y[i+1],
#        a2[i] is a lower (1 - alpha/2)-confidence bound
#        for F(x).
#    b2 : For 1 <= i <= n and y[i] <= x < y[i+1],
#        b2[i] is an upper (1 - alpha/2)-confidence bound
#        for F(x).
#    a2.naive, b2.naive : These are the usual Clopper-
#        Pearson (1 - alpha/2)-confidence bounds for F(x),
#        assuming that X is an i.i.d. sample from F, so
#        #{i <= n : X[i] <= x} follows Bin(n,F(x)).
# 
# If display==TRUE, the procedure plots six different
# step functions:
#    empirical distribution function (cyan, thin),
#    moment-matching estimator of F (blue, thin),
#    one-sided confidence bounds a1, b1 (blue)
#    two-sided confidence bounds a2, b2 (black)
# In case of display.all==TRUE, the confidence bounds
# a2.naive, b2.naive are displayed as well (cyan).
{
	tmp <- order(X)
	X <- X[tmp]
	R <- R[tmp]
	N <- table(factor(R,levels=1:k))
	n <- length(X)
	cbs <- ConfBandPW(N,alpha)
	cbs[,1] <- c(-Inf,X)
	if (display || display.all){
		Xs <- c(xlim[1],rep(X,each=2),xlim[2])
		FhatM <- FhatM(N)
		plot(Xs,rep(0:n,each=2)/n,type="l",col='cyan',
			lwd=1,
			xlab=expression(italic(x)),
			ylab=expression(italic(F(x))))
		if (display.all){
			lines(Xs,rep(cbs[,6],each=2),
				lwd=2,col="cyan")
			lines(Xs,rep(cbs[,7],each=2),
				lwd=2,col="cyan")			
		}
		lines(Xs,rep(FhatM,each=2),col='blue')
		lines(Xs,rep(cbs[,2],each=2),lwd=2,col="blue")
		lines(Xs,rep(cbs[,3],each=2),lwd=2,col="blue")
		lines(Xs,rep(cbs[,4],each=2),lwd=2,col="black")
		lines(Xs,rep(cbs[,5],each=2),lwd=2,col="black")
	}
	return(conf.bounds=cbs)
}


ConfBandPW <- function(N,alpha=0.05,
	display=FALSE,
	display.all=FALSE)
# This procedure is the essential ingredient for the
# function ConfBandPW_Data():
# For a given vector N of k frequencies
#    N[r] = #{i <= n : R[i] = r},  1 <= r <= k, 
# (1 - alpha)-confidence intervals for F(x) are given by
#    [a1(H(x)), 1],
#    [0, b1(H(x))],
#    [a2(H(x)), b2(H(x))]
# with
#    H(x) := #{i : X_i <= x}
# and certain bounds a1(y), b1(y), a2(y), b2(y) depending
# only on y in {0,1,...,n}, N and alpha.
# This procedure computes all these bounds and returns them
# as one table. The graphical output (if desired) is
# analogous to the output of ConfBandPW_Data().
{
	n <- sum(N)
	a1 <- rep(0,n+1)
	a2 <- rep(0,n+1)
	a2.naive <- rep(0,n+1)
	b1 <- rep(1,n+1)
	b2 <- rep(1,n+1)
	b2.naive <- rep(1,n+1)
	for (y in 0:n){
		if (y > 0){
			a1[y+1] <- LocalINV(N,y,
				gamma=alpha,  lower.tail=FALSE)[1]
			a2[y+1] <- LocalINV(N,y,
				gamma=alpha/2,lower.tail=FALSE)[1]
		}
		if (y < n){
			b1[y+1] <- LocalINV(N,y,gamma=alpha  )[2]
			b2[y+1] <- LocalINV(N,y,gamma=alpha/2)[2]
		}
		tmp <- binom.test(y,n,conf.level=1-alpha)$
				conf.int
		a2.naive[y+1] <- tmp[1]
		b2.naive[y+1] <- tmp[2]
	}
	conf.bounds <- cbind(y=0:n,a1,b1,a2,b2,
		a2.naive,b2.naive)
	if (display || display.all){
		yn <- c(0,rep(1:n,each=2),n+1)
		FhatM <- FhatM(N)
		par(mai=c(0.8,0.9,0.02,0.02))
		plot(yn,rep((0:n)/n,each=2),type="l",
			col="cyan",
			xlab=expression(italic(y)),
			ylab=expression(italic(list(a(N,y),b(N,y)))))
		if (display.all){
			lines(yn,rep(a2.naive,each=2),
				lwd=2,col="cyan")
			lines(yn,rep(b2.naive,each=2),
				lwd=2,col="cyan")			
		}
		lines(yn,rep(FhatM,each=2),col="blue")
		lines(yn,rep(a1,each=2),lwd=2,col="blue")
		lines(yn,rep(b1,each=2),lwd=2,col="blue")
		lines(yn,rep(a2,each=2),lwd=2,col="black")
		lines(yn,rep(b2,each=2),lwd=2,col="black")
	}
	return(conf.bounds)
}


# Auxiliary functions:

LocalCDF <- function(N,p,lower.tail=TRUE)
# Computes the distribution function (if lower.tail ==
#  TRUE) or left-continuous survival function (if
#  lower.tail == FALSE) of the random variable
#    Y = sum_{r=1}^k Y_r
# with independent random variables
#    Y_r ~ Bin(N[r],B_r(p)), 1 <= r <= k .
# Output is a vector GG of length n+1, where n = sum(N) and
#   GG[i] = P(Y <= i-1)  if lower.tail == TRUE,
#   GG[i] = P(Y >= i-1)  if lower.tail == FALSE.
{
	k <- length(N)
	gg <- 1
	for (r in 1:k){
		pr <- pbeta(p,r,k+1-r)
		gg <- convolve(gg,
			dbinom(N[r]:0,N[r],pr),type='open')
	}
	if (lower.tail){
		GG <- cumsum(gg)
	}
	else{
		nn <- length(gg)
		GG <- cumsum(gg[nn:1])
		GG <- GG[nn:1]
	}
	return(GG)
}


LocalINV <- function(N,y,gamma=0.5,tol=10^(-7),
	lower.tail=TRUE)
# For an integer y in {0,1,...,n-1}, where n = sum(N),
# this procedure provides a pair (a,b) of numbers
# approximating the unique solution p in (0,1) of the
# equation
#    G_p(y) = gamma.
# Here G_p is the c.d.f. of the random variable Y
# described in LocalCDF. Precisely,
#    0 < a <= p <= b < 1
# with
#    logit(b) - logit(a) <= tol,
#    logit(G_a(y)) - logit(G_b(y)) <= tol.
# If lower.tail == FALSE, we require that 0 < y <= n and
# consider G_p(y) = P(Y >= y), so
#    logit(G_b(y)) - logit(G_a(y)) <= tol.
{
	n <- sum(N)
	if (lower.tail){
		if (y == n){
			return(c(NA,NA))
		}
		a <- 0
		Ga <- 1
		b <- 1
		Gb <- 0
		while (logit(b) - logit(a) > tol ||
			logit(Ga) - logit(Gb) > tol){
			p <- (a + b)/2
			Gp <- LocalCDF(N,p)[y+1]
			if (Gp > gamma){
				a <- p
				Ga <- Gp
			}
			else{
				b <- p
				Gb <- Gp
			}
		}
	}
	else{
		if (y == 0){
			return(c(NA,NA))
		}
		a <- 0
		Ga <- 0
		b <- 1
		Gb <- 1
		while (logit(b) - logit(a) > tol ||
			logit(Gb) - logit(Ga) > tol){
			p <- (a + b)/2
			Gp <- LocalCDF(N,p,lower.tail=FALSE)[y+1]
			if (Gp > gamma){
				b <- p
				Gb <- Gp
			}
			else{
				a <- p
				Ga <- Gp
			}
		}
	}
	return(c(a,b))
}


logit <- function(u)
{
	return(log(u/(1-u)))
}


############################################################
### Kolmogorov-Smirnov confidence bands                  ###
############################################################

# Required: Function FhatM(.) from Estimation.R

ConfBandKS_Data <- function(X,R,k,alpha=0.05,
	conf.bounds=NULL,
	tol=10^(-7),
	mcsim=39999,
	xlim=c(1.1*min(X)-0.1*max(X), 1.1*max(X)-0.1*min(X)),
	display=FALSE)
# A Kolmogorov-Smirnov confidence band for F based on the
# moment-matching estimator FhatM(.) has also the property
# that it depends only on the order statistics of X and
# the tuple N of frequencies
#    N[r] = #{i <= n : R[i] = r}.
# Precisely, if
#    0 = FM_0 < FM_1 < FM_2 < ... < FM_n = 1
# are the n+1 different values of FhatM(.) on the
# intervals
#    (-Inf,X_(1)], (X_(1),X_(2)], ..., (X_(n),Inf] ,
# then the (1 - alpha)-confidence band for F has
# boundaries
#    max(FM_i - kappa, 0)  and  min(FM_i + kappa, 1)
# on (X_(i), X_(i+1)], 0 <= i <= n.
# This procedure determines a Monte Carlo estimate of the
# (1 - alpha)-quantile kappa of
#    sup_x |FhatM(x) - F(x)| ,
# conditional on N. In addition to kappa it returns (and
# displays, if desired) a table with four columns:
#    x : order statistics X_(y), 0 <= y <= n,
#    FhatM : corresponding values FM_y
#    a : corresponding lower bounds max(FH_y - kappa, 0)
#    b : corresponding upper bounds min(FH_y + kappa, 1)
{
	tmp <- order(X)
	X <- X[tmp]
	R <- R[tmp]
	N <- table(factor(R,levels=1:k))
	n <- length(X)

	FhatM <- FhatM(N,tol=tol,display=FALSE)
	Tv <- rep(NA,mcsim)
	for (s in 1:mcsim){
		X.sim <- sort(rbeta(n,R,k+1-R))
		Tv[s] <- max(max(FhatM[2:(n+1)] - X.sim),
		             max(X.sim - FhatM[1:n]))
	}
	Tv <- sort(Tv)
	kappa <- Tv[ceiling((mcsim+1)*(1 - alpha))]
	an <- pmax(FhatM - kappa, 0)
	bn <- pmin(FhatM + kappa, 1)
	if (display){
		xn <- c(xlim[1],rep(X,each=2),xlim[2])
		plot(xn,rep((0:n)/n,each=2),type="l",
			col="cyan",
			xlab=expression(italic(x)),
			ylab=expression(italic(F(x))))
		lines(xn,rep(FhatM,each=2),lwd=2,col="blue")
		lines(xn,rep(an,each=2),lwd=2,col="black")
		lines(xn,rep(bn,each=2),lwd=2,col="black")
	}
	conf.bounds <- cbind(x=c(-Inf,X),
		FhatM=FhatM,a=an,b=bn)
	return(list(conf.bounds=conf.bounds,
		kappa=kappa,conf.level=1-alpha))
}

# In simulation studies involving KS bands for many data
# sets (X,R) with identical frequency vector N, it is more
# economical to determine the critical value kappa and the
# 3*(n+1) relevant values of the estimator FhatM and the
# bounds max(FhatM-kappa,0), min(FhatM+kappa,1) only once:

ConfBandKS <- function(N,alpha=0.05,
	tol=10^(-7),
	mcsim=39999,
	display=FALSE)
# Similar to ConfBandKS_Data(), but this time only the
# frequency vector N is fixed. The bounds are computed
# (and displayed) for a hypothetical data vector X = 1:n.
{
	n <- sum(N)
	k <- length(N)
	R <- rep(1:k,times=N)
	FhatM <- FhatM(N,tol=tol,display=FALSE)
	Tv <- rep(NA,mcsim)
	for (s in 1:mcsim){
		X <- sort(rbeta(n,R,k+1-R))
		Tv[s] <- max(max(FhatM[2:(n+1)] - X),
		             max(X - FhatM[1:n]))
	}
	Tv <- sort(Tv)
	kappa <- Tv[ceiling((mcsim+1)*(1 - alpha))]
	an <- pmax(FhatM - kappa, 0)
	bn <- pmin(FhatM + kappa, 1)
	if (display){
		yn <- c(0,rep(1:n,each=2),n+1)
		plot(yn,rep((0:n)/n,each=2),type="l",
			col="cyan",
			xlab="y",ylab="")
		lines(yn,rep(FhatM,each=2),lwd=2,col="blue")
		lines(yn,rep(an,each=2),lwd=2,col="black")
		lines(yn,rep(bn,each=2),lwd=2,col="black")
	}
	conf.bounds <- cbind(y=0:n,FhatM=FhatM,a=an,b=bn)
	return(list(conf.bounds=conf.bounds,
		kappa=kappa,conf.level=1-alpha))
}

ConfBandKS0 <- function(n,alpha=0.05,
	mcsim=39999,
	display=FALSE)
# For comparison purposes, this procedure returns a
# MC estimate for the (1 - alpha)-quantile of the
# classical KS-statistic for a sample of size n and the
# resulting confidence bounds on n+1 intervals determined
# by the order statistics of a sample of size n.
{
	Fhat <- (0:n)/n
	Tv <- rep(NA,mcsim)
	for (s in 1:mcsim){
		X <- sort(runif(n))
		Tv[s] <- max(max(Fhat[2:(n+1)] - X),
		             max(X - Fhat[1:n]))
	}
	Tv <- sort(Tv)
	kappa <- Tv[ceiling((mcsim+1)*(1 - alpha))]
	an <- pmax(Fhat - kappa, 0)
	bn <- pmin(Fhat + kappa, 1)
	if (display){
		yn <- c(0,rep(1:n,each=2),n+1)
		plot(yn,rep((0:n)/n,each=2),type="l",
			col="cyan",
			xlab="y",ylab="")
		lines(yn,rep(Fhat,each=2),lwd=2,col="blue")
		lines(yn,rep(an,each=2),lwd=2,col="black")
		lines(yn,rep(bn,each=2),lwd=2,col="black")
	}
	conf.bounds <- cbind(y=0:n,Fhat=Fhat,a=an,b=bn)
	return(list(conf.bounds=conf.bounds,
		kappa=kappa,alpha=alpha))
}
