############################################################
### Point estimation of the distribution function F      ###
### Lutz Duembgen, Ehsan Zamanzadeh, July 2018           ###
############################################################

# The underlying data are two vectors X and R of length n,
# where (X[i],R[i]), 1 <= i <= n, are assumed to be
# independent with R[i] in {1,...,k} and
#   P(X[i] <= x | R[i] = r) = pbeta(F(x),r,k+1-r) .
# Here F is the unknown distribution function to be
# estimated.

# Implemented are four point estimators of the c.d.f. F:
# - FhatN: Naive empirical cdf (cyan)
# - FhatS: Stratified empirical cdf (red)
# - FhatM: Moment-matching estimator (blue)
# - FhatL: Maximum conditional likelihood (black)


Fhat_Data <- function(X,R,k,tol=10^(-7),
	display=TRUE)
# Computation of the four estimators of F
{
	# Sorting the observations w.r.t. X:
	tmp <- order(X)
	Xs <- X[tmp]
	Rs <- R[tmp]
	n <- length(X)
	
	# Computing frequencies of ranks and rank-wise
	# empirical distribution functions:
	Fhat <- matrix(0,n,k)
	Nv <- rep(0,k)
	for (r in 1:k){
		tmp <- (Rs == r)
		Nv[r] <- sum(tmp)
		if (Nv[r] > 0){
			Fhat[,r] <- cumsum(tmp)/Nv[r]
		}
	}
	
	# Naive empirical c.d.f.:
	FhatN <- (1:n)/n
	
	# Stratified empirical c.d.f.:
	FhatS <- cumsum(1/Nv[Rs]) / sum(Nv > 0)
	
	# Moment matching estimator:
	kv <- 1:k
	FhatM <- rep(1,n)
	a <- 0
	nFa <- 0
	for (i in 1:(n-1)){
		b <- 1
		nFb <- n
		while (logit(b) - logit(a) > tol ||
			logit(nFb/n) - logit(nFa/n) > tol){
			c <- (a + b)/2
			nFc <- sum(Nv*pbeta(c,kv,k+1-kv))
			if (nFc < i){
				a <- c
				nFa <- nFc
			}else{
				b <- c
				nFb <- nFc
			}
		}
		FhatM[i] <- (a+b)/2
	}
	
	# Maximum likelihood estimator:
	FhatL <- rep(1,n)
	a <- 0
	Da <- Inf
	for (i in 1:(n-1)){
		b <- 1
		Db <- -Inf
		while (logit(b) - logit(a) > tol ||
			Da - Db > tol){
			c <- (a + b)/2
			Dc <- sum(Nv*
				dbeta(c,kv,k+1-kv)/
				pbeta(c,kv,k+1-kv)/
				pbeta(1-c,k+1-kv,kv) *
				(Fhat[i,] - pbeta(c,kv,k+1-kv)))
			if (Dc > 0){
				a <- c
				Da <- Dc
			}else{
				b <- c
				Db <- Dc
			}
		}
		FhatL[i] <- (a+b)/2
	}
	
	Ftable <- data.frame(xs=Xs,
			FhatN=FhatN,
			FhatS=FhatS,
			FhatM=FhatM,
			FhatL=FhatL)
	
	if (display){
		PlotFhat(Ftable)
	}
	
	return(Ftable)
}


logit <- function(u)
{
	return(log(u/(1-u)))
}


PlotFhat <- function(Ftable,
	xlim=c(matrix(c(1.1,-0.1,-0.1,1.1),2,2) %*%
		range(Ftable$xs)))
{
	xs <- c(xlim[1],rep(Ftable$xs,each=2),xlim[2])
	FhatN <- c(0,0,rep(Ftable$FhatN,each=2))
	FhatS <- c(0,0,rep(Ftable$FhatS,each=2))
	FhatM <- c(0,0,rep(Ftable$FhatM,each=2))
	FhatL <- c(0,0,rep(Ftable$FhatL,each=2))
	
	par(mfrow=c(1,1))
	plot(xs,FhatN,type="l",col="cyan",
		xlab="t",ylab="F(t)")
	lines(xs,FhatS,lwd=3,col="red")
	lines(xs,FhatM,lwd=2,col="blue")
	lines(xs,FhatL,lwd=1,col="black")
}


######################################################
### Auxiliary programs for simulation studies etc. ###
######################################################

FhatSL_Data <- function(X,R,N,n,k,tol=10^(-7))
# Similar functionality as Fhat_Data (without graphical
# display): Computation of FhatS and FhatL, assuming
# X to be sorted already and N, n to be specified.
{
	# Computing frequencies of ranks and rank-wise
	# empirical distribution functions:
	Fhat <- matrix(0,n,k)
	for (r in 1:k)
	{
		if (N[r] > 0)
		{
			tmp <- (R == r)
			Fhat[,r] <- cumsum(tmp)/N[r]
		}
	}
	
	# Stratified empirical c.d.f.:
	FhatS <- cumsum(1/N[R])/sum(N > 0)
	
	# Maximum likelihood estimator:
	kv <- 1:k
	FhatL <- rep(1,n)
	a <- 0
	Da <- Inf
	for (i in 1:(n-1))
	{
		b <- 1
		Db <- -Inf
		while (logit(b) - logit(a) > tol ||
			Da - Db > tol)
		{
			c <- (a + b)/2
			Dc <- sum(N*
				dbeta(c,kv,k+1-kv)/
				pbeta(c,kv,k+1-kv)/
				pbeta(1-c,k+1-kv,kv) *
				(Fhat[i,] - pbeta(c,kv,k+1-kv)))
			if (Dc > 0)
			{
				a <- c
				Da <- Dc
			}
			else
			{
				b <- c
				Db <- Dc
			}
		}
		FhatL[i] <- (a+b)/2
	}
	
	Ftable <- cbind(FhatS=FhatS,FhatL=FhatL)
	
	return(Ftable)
}


FhatL_Data <- function(X,R,N,n,k,tol=10^(-7))
# Computation of FhatL (without graphical display),
# assuming X to be sorted already and N, n to be specified.
{
	# Computing frequencies of ranks and rank-wise
	# empirical distribution functions:
	Fhat <- matrix(0,n,k)
	for (r in 1:k)
	{
		if (N[r] > 0)
		{
			tmp <- (R == r)
			Fhat[,r] <- cumsum(tmp)/N[r]
		}
	}
	
	# Maximum likelihood estimator:
	kv <- 1:k
	FhatL <- rep(1,n)
	a <- 0
	Da <- Inf
	for (i in 1:(n-1))
	{
		b <- 1
		Db <- -Inf
		while (logit(b) - logit(a) > tol ||
			Da - Db > tol)
		{
			c <- (a + b)/2
			Dc <- sum(N*
				dbeta(c,kv,k+1-kv)/
				pbeta(c,kv,k+1-kv)/
				pbeta(1-c,k+1-kv,kv) *
				(Fhat[i,] - pbeta(c,kv,k+1-kv)))
			if (Dc > 0)
			{
				a <- c
				Da <- Dc
			}
			else
			{
				b <- c
				Db <- Dc
			}
		}
		FhatL[i] <- (a+b)/2
	}
	
	return(FhatL)
}


FhatM <- function(N,tol=10^(-7),display=FALSE)
# The moment-matching estimator FhatM(.) of F(.) depends
# only on the order statistics of X and the tuple N of
# frequencies
#    N[r] = #{i <= n : R[i] = r}.
# Precisely, if
#    -Inf = X_(0) < X(1) < ... < X_(n) < X_(n+1) = Inf
# denote the augmented order statistics of X, then for
# 0 <= y <= n and X_(y) < x <= X_(y+1),
#    FhatM(x) = 0  if y = 0,
#    FhatM(x) = 1  if y = n,
# and otherwise
#    FhatM(x) = unique parameter p in (0,1) such that
#               the random variable Y described in
#               LocalCDF() satisfies  E(Y) = y.
# This procedure returns (and displays, if desired) a
# vector with these n+1 possible values of FhatM.
{
	k <- length(N)
	n <- sum(N)
	FhatM <- rep(0,n+1)
	FhatM[n+1] <- 1
	rv <- 1:k
	for (i in 1:(n-1)){
		if (i == 1){
			a <- 0
			nFa <- 0
		}
		b <- 1
		nFb <- n
		while (logit(b) - logit(a) > tol ||
			logit(nFb/n) - logit(nFa/n) > tol){
			c <- (a + b)/2
			nFc <- sum(N*pbeta(c,rv,k+1-rv))
			if (nFc < i){
				a <- c
				nFa <- nFc
			}else{
				b <- c
				nFb <- nFc
			}
		}
		FhatM[i+1] <- (a+b)/2
	}
	
	if (display){
		par(mai=c(0.8,0.9,0.02,0.02))
		xn <- c(0,rep(1:n,each=2),n+1)
		plot(xn,rep((0:n)/n,each=2),type="l",
			col="cyan",
			xlab=expression(italic(y)),
			ylab=expression(italic({widehat(F)^M}(X[(y)]))))
		lines(xn,rep(FhatM,each=2),lwd=2,col="black")
	}

	return(FhatM)
}