############################################################
### Inference on a Distribution Function                 ###
### from Ranked Set Samples                              ###
### Lutz Duembgen, Ehsan Zamanzadeh, July 2018           ###
############################################################

# This R script contains various examples illustrating
# the methods in the paper mentioned above.

# The underlying data are two vectors X and R of length n,
# where (X[i],R[i]), 1 <= i <= n, are assumed to be
# independent with R[i] in {1,...,k} and
#   P(X[i] <= x | R[i] = r) = pbeta(F(x),r,k+1-r) .
# Here F is the unknown distribution function to be
# estimated.

# List of R files:

# Estimation.R:  Point estimators of F
# Simulations.R: Simulation of RSS and JPS data
# ConfBands.R:   Pointwise and simultaneous confidence
#                bands for F
# MonteCarlo.R:  Various simulation programs.

source("Estimation.R")
source("Simulations.R")
source("ConfBands.R")
source("MonteCarlo.R")


############################################################
### Point estimators based on RSS data                   ###
############################################################

### Balanced case ###

# Number of groups:
k <- 5
# Common group size:
Nk <- 40
# Vector of group sizes:
N <- rep(Nk,k)

# Generate Gaussian RSS data:
# Perfect ranking:
data <- SimDataRSS(N)
# Imperfect ranking (Dell-Clutter model):
# data <- SimDataRSS.imp(N,rho=0.9)
X <- data$X
R <- data$R
# Data matrix:
cbind(X,R)

# Point estimation of F:
FhatX <- Fhat_Data(X,R,k) 
# Colour code of methods in graphical output:
# - cyan:  naive empirical
# - red:   stratified empirical
# - blue:  moment-matching
# - black: maximum conditional likelihood
# Depict also true cdf F:
xx <- seq(1.1*min(X)-0.1*max(X),1.1*max(X)-0.1*min(X),
	length.out=401)
lines(xx,pnorm(xx),col='green')
# Table of four estimators:
FhatX
# Column 1:    order statistics of X
# Columns 2-5: four estimators of F at the
#              order statistics of X,


### Unbalanced case ###

# Vector of group sizes:
N <- c(70,50,40,30,10)
k <- length(N)

# Generate Gaussian RSS data:
# Perfect ranking:
data <- SimDataRSS(N)
# Imperfect ranking (Dell-Clutter model):
# data <- SimDataRSS.imp(N,rho=0.9)
X <- data$X
R <- data$R
# Data matrix:
cbind(X,R)

# Point estimation of F:
FhatX <- Fhat_Data(X,R,k)
# Depict also true cdf F:
xx <- seq(1.1*min(X)-0.1*max(X),1.1*max(X)-0.1*min(X),
	length.out=401)
lines(xx,pnorm(xx),col='green')
# Table of four estimators:
FhatX


# Display of moment-matching estimator alone for the
# given vector N (with pseudo observations X[i] = i):
FhatM.list <- FhatM(N,display=TRUE)
cbind('i'=0:sum(N),'FhatM(X_(i))'=FhatM.list)


############################################################
### Pointwise confidence bands for F based on RSS data   ###
############################################################

# Vector of group sizes:
N <- c(40,30,0,20,10)
# More extreme example:
# N <- c(60,30,10,0,0)
k <- length(N)

# Display of pointwise 95%-confidence band for F based
# on group sizes N (with pseudo-observations X[i] = i):   
CB0 <- ConfBandPW(N,display=TRUE)
CB0 <- ConfBandPW(N,display.all=TRUE)
# The step functions are:
# - naive empirical distribution function (cyan, thin)
# - moment-matchin estimator (blue, thin)
# - one-sided 95%-confidence bounds (blue)
# - two-sided 95%-confidence bounds
#   i.e. one-sided 97.5%-confidence bounds (black)
# - naive two-sided 95%-confidence bounds
#   based on binomial model for empirical c.d.f. (cyan)
#   in case of display.all==TRUE

# Table of these bounds:
CB0

# Applied to data:
data <- SimDataRSS(N)
X <- data$X
R <- data$R
CB <- ConfBandPW_Data(X,R,k,display=TRUE)
CB <- ConfBandPW_Data(X,R,k,display.all=TRUE)

# Depict also true cdf F:
xx <- seq(1.1*min(X)-0.1*max(X),1.1*max(X)-0.1*min(X),
	length.out=401)
lines(xx,pnorm(xx),col='green')


############################################################
### Kolmogorov-Smirnov bands for F based on RSS data     ###
############################################################

# Vector of group sizes:
k <- 5
Nk <- 20
N <- rep(Nk,k)

# Display of simultaneous 95%-confidence band for F based
# on group sizes N (with pseudo-observations X[i] = i):   
CB0 <- ConfBandKS(N,display=TRUE)

# Table of point estimator and confidence bounds,
# critical value, confidence level:
CB0


# Applied to data:
data <- SimDataRSS(N)
X <- data$X
R <- data$R
CB <- ConfBandKS_Data(X,R,k,display=TRUE)

# Depict also true cdf F:
xx <- seq(1.1*min(X)-0.1*max(X),1.1*max(X)-0.1*min(X),
	length.out=401)
lines(xx,pnorm(xx),col='green')

# Table of estimator and confidence bounds,
# critical value, confidence level:
CB


############################################################
### Error analyses for RSS sampling from a Gaussian      ###
### population with ranking errors (Dell-Clutter model)  ###
############################################################

# Ranking precision (i.e. the correlation between true
#  data and proxies):
rho <- 0.90
# Group sizes:
N <- c(70,30,0,30,70)
k <- length(N)

# Analyze precision of all four estimators in terms
# of bias and root mean squared error:
ErrorAnalysisRSS.imp(N,rho,mcsim=100)
ErrorAnalysisRSS.imp(N,rho,mcsim=2000)
	# This will take a while...


############################################################
### RSS samples from a finite population                 ###
############################################################

### Simulated data ###

# Simulate a population and data:
N <- c(70,50,30)
k <- length(N)
n <- sum(N)

# "True" population values:
m <- 10
Y <- rnorm(n*k*m)
# Proxy values:
rho <- 0.95
Y.proxy <- rho*Y + sqrt(1 - rho^2)*rnorm(n*k*m)

# Simulate an RSS sample from that population:
data <- SimDataRSS.finpop(Y,Y.proxy,N)
X <- data$X
R <- data$R
# Proportion of false ranks:
data$prop.false.ranks

# Illustrate point estimator FhatM for this
# particular sample:
FhatM.finpop(X,N,Y)
# The same function, but with FhatM precomputed:
phi <- FhatM(N)
FhatM.finpop(X,N,Y,phi=phi)

# Precomputing the pointwise 95%-confidence intervals:
ab.pw <- ConfBandPW(N)[,4:5]
# Display of estimator and pointwise 95%-confidence band:
FhatM.finpop(X,N,Y,phi=phi,ab.pw=ab.pw)

# Precomputing the values of FhatM and the simultaneous
# 95%-confidence intervals:
CB0 <- ConfBandKS(N)
str(CB0)
kappa <- CB0$kappa
ab.sim <- CB0$conf.bounds[,3:4]
# Display of estimator and simultaneous 95%-confidence
# band:
FhatM.finpop(X,N,Y,phi=phi,ab.sim=ab.sim)
FhatM.finpop(X,N,Y,phi=phi,kappa=kappa)

# Display of everything:
FhatM.finpop(X,N,Y,phi=phi,ab.pw=ab.pw,kappa=kappa)


### Real data ###

# Swiss municipalities:
ds <- read.table('Municip_CH_2015.txt',header=TRUE)
str(ds)

# Variable of interest: log10-population size in 2015
Y <- log10(ds[,3])
# Proxy values: log10-population size in 2010
Y.proxy <- log10(ds[,3]/(1 + ds[,4]/100))

# We are thus simulating the situation that by the end of
# 2015 the municipalities' population sizes are not yet
# available as a complete list, although one could ask
# for them individually. What is available, though, are
# the population sizes in 2010.

plot(Y.proxy,Y,
	xlab='log10(# inhabitants 2015)',
	ylab='log10(# inhabitants 2010)')

N <- c(100,70,40)
k <- length(N)
n <- sum(N)

data <- SimDataRSS.finpop(Y,Y.proxy,N)
X <- data$X
R <- data$R
# Proportion of false ranks:
data$prop.false.ranks

# Precomputations:
phi <- FhatM(N)
ab.pw <- ConfBandPW(N)[,4:5]
CB0 <- ConfBandKS(N,mcsim=99999)
kappa <- CB0$kappa
ab.sim <- CB0$conf.bounds[,3:4]

# Display of everything:
FhatTable <- FhatM.finpop(X,N,Y,
	phi=phi,ab.pw=ab.pw,kappa=0.0812,
	xlab=expression(plain(log[10](x))),
	ylab=expression(plain(F(x))))
# Display of everything but FhatM:
FhatTable <- FhatM.finpop(X,N,Y,
	phi=phi,ab.pw=ab.pw,kappa=0.0812,
	xlab=expression(plain(log[10](x))),
	ylab=expression(plain(F(x))),
	displayFhatM=FALSE)
FhatTable


### Simulation study of performance ###

# For a given population (given by Y and Y.proxy)
# estimate and depict
# - bias and RMSE of FhatM
# - average width of pointwise and simultaneous
#   confidence intervals
# - coverage probabilities of pointwise
#   confidence intervals
# In addition, the test level (1 - confidence level)
# of the simultaneous confidence band is estimated.

N <- c(100,70,40)
results <- ErrorAnalysisRSS.finpop(Y,Y.proxy,N,
	phi=phi,ab.pw=ab.pw,kappa=kappa,
	xlab=expression(plain(log[10](x))),
	mcsim=1000)
# Estimated test level (1 - confidence level) of
# simultaneous confidence band:
results$cov.err.sim
