############################################################
### Monte Carlo simulations for various JPS/RSS methods  ###
### Lutz Duembgen, Ehsan Zamanzadeh, July 2018           ###
############################################################


############################################################
### Error analyses for RSS sampling from a Gaussian      ###
### population with ranking errors (Dell-Clutter model)  ###
############################################################

ErrorAnalysisRSS.imp <- function(N,rho=1,
	xx=qnorm(seq(0.05,0.95,by=0.05)),
	mcsim=1000,display=TRUE)
# In the setting of SimDataRSS.imp(), this procedure
# simulates mcsim such samples and analyses the precision
# of the four estimators FhatN, FhatS, FhatM and FhatL
# in terms of bias and root mean squared error.
{
	# Initialize output:
	BIAS <- matrix(0,length(xx),4)
	dimnames(BIAS)[[2]] <- c('N','S','M','L')
	RMSE <- matrix(0,length(xx),4)
	dimnames(RMSE)[[2]] <- c('N','S','M','L')
	
	# Precomputation of FhatN and FhatM:	
	k <- length(N)
	n <- sum(N)
	FhatN <- (0:n)/n
	FhatM <- FhatM(N)
	
	# Rank vector:
	R <- rep(1:k,times=N)
	
	# Compute c.d.f. F at points in xx:
	Fxx <- pnorm(xx)
	
	# MC simulations:
	for (s in 1:mcsim){
		if (rho < 1){
			# Concomitant observations:
			Y <- qnorm(rbeta(n,R,k+1-R))
			# True observations:
			X <- rho*Y + rnorm(n,0,sqrt(1 - rho^2))
		}else{
			X <- qnorm(rbeta(n,R,k+1-R))
		}
		tmp1 <- order(X)
		Xs <- X[tmp1]
		Rs <- R[tmp1]
		tmp2 <- FhatSL_Data(Xs,Rs,N,n,k)
		Ftable <- cbind(FhatN,c(0,tmp2[,1]),FhatM,c(0,tmp2[,2]))
		Xe <- c(-Inf,Xs,Inf)
		iX <- 1
		for (j in 1:length(xx)){
			while (Xe[iX+1] <= xx[j]){
				iX <- iX+1
			}
			BIAS[j,] <- BIAS[j,] +  Ftable[iX,] - Fxx[j]
			RMSE[j,] <- RMSE[j,] + (Ftable[iX,] - Fxx[j])^2
		}
		if (s %% 100 == 0){
			print(paste(s,'simulations done...'))
		}
	}
	BIAS <- BIAS/mcsim
	RMSE <- sqrt(RMSE/mcsim)

	# Utilize symmetry if possible:
	if (sum(N != N[k:1]) == 0 &&
		max(abs(xx + xx[length(xx):1])) < 10^(-10)){
		BIAS <- (BIAS - BIAS[length(xx):1,])/2
		RMSE <- (RMSE + RMSE[length(xx):1,])/2
	}

	
	if (display){
		par(cex=1.5,mai=c(0.8,0.8,0.05,0.05),
			mgp=c(2.3,1,0),mfrow=c(2,1))
		plot(range(xx),c(0,0),type='l',col='green',
			xlab=expression(plain(x)),
			ylab=expression(plain(BIAS(x))),
			ylim=range(BIAS))
		lines(xx,BIAS[,1],lwd=2,col='cyan')
		lines(xx,BIAS[,1],lty=3)
		lines(xx,BIAS[,2],lwd=2,col='red')
		lines(xx,BIAS[,2],lty=3)
		lines(xx,BIAS[,3],lwd=2,col='blue')
		lines(xx,BIAS[,3],lty=3)
		lines(xx,BIAS[,4],lwd=2,col='black')
		plot(range(xx),c(0,0),type='l',col='green',
			xlab=expression(plain(x)),
			ylab=expression(plain(RMSE(x))),
			ylim=c(0,max(RMSE)))
		lines(xx,RMSE[,1],lwd=2,col='cyan')
		lines(xx,RMSE[,1],lty=3)
		lines(xx,RMSE[,2],lwd=2,col='red')
		lines(xx,RMSE[,2],lty=3)
		lines(xx,RMSE[,3],lwd=2,col='blue')
		lines(xx,RMSE[,3],lty=3)
		lines(xx,RMSE[,4],lwd=2,col='black')
	}
	return(list(BIAS=cbind('x'=xx,BIAS),
		RMSE=cbind('x'=xx,RMSE)))
}


############################################################
### Error analyses for RSS sampling from a finite        ###
### population as described in Section 4 of              ###
### Duembgen and Zamanzadeh (2018)                       ###
############################################################


SimDataRSS.finpop <- function(Y,Y.proxy=Y,N)
# Consider a finite population with a numerical feature,
# represented by a numerical vector Y, and a vector
# Y.proxy of proxys for Y.
# This procedure produces a ranked set sample as follows:
# One chooses n = sum(N) disjoint subsets J(1), ..., J(n)
# of the index set {1,2,...,length(Y)}, each of size k.
# Now the elements of J(i) are ranked with respect to
# Y.proxy, and then one chooses X[i] to be the value Y[j]
# of the index j in J(i) having rank R[i].
# Here R[1], R[2], ... R[n] are ranks in {1,...,k} such
# that rank r appears N[r] times.
{
	# Preprocess Y:
	nY <- length(Y)
	n <- sum(N)
	k <- length(N)
	if (n*k > nY){
		return('Population too small!')
	}
	
	# Generate rank vector R and initialize
	# observation vector X:
	R <- rep(1:k,times=N)
	X <- rep(NA,n)
	
	# Sampling:
	pi <- sample(nY)
	prop.false.ranks <- 0
	for (i in 1:n){
		tmp <- pi[(k*i-k+1):(k*i)]
		Rtmp <- rank(Y.proxy[tmp],ties.method='first')
		Jtmp <- which(Rtmp==R[i])
		X[i] <- Y[tmp[Jtmp]]			
		RtmpA <- sum(Y[tmp] < X[i])
		RtmpB <- sum(Y[tmp] <= X[i])
		prop.false.ranks <- prop.false.ranks +
			(RtmpA >= R[i] || RtmpB < R[i])
	}
	prop.false.ranks <- prop.false.ranks/n
	return(list(X=X,R=R,prop.false.ranks=prop.false.ranks))
}

FhatM.finpop <- function(X,N,Y,
	phi=FhatM(N),ab.pw=NULL,kappa=NULL,ab.sim=NULL,
	display=TRUE,displayFhatM=TRUE,
	xlab=NULL,ylab=NULL,
	xlim=c(1.1*min(Y)-0.1*max(Y),1.1*max(Y)-0.1*min(Y)))
# For RSS data (X,R) sampled from a finite population Y
# as in SimDataRSS.finpop() with stratum sizes given by N,
# this procedure computes (and displays, if display==TRUE)
# the
# - true distribution function F (green)
# - moment-matching estimator FhatM (blue)
# - the pointwise 95%-confidence band (blue)
#    if ab.pw!=NULL
# - the simultaneous 95%-confidence band (black)
#    if ab.sim!=NULL or kappa!=NULL
# Note that for all these procedures the rank vector R
# is not needed, only the tuple N of stratum sizes.
{
	n <- length(X)
	Xs <- sort(X)
	nY <- length(Y)
	Ys <- sort(Y)
	# Graphical display:
	if (display){
		xlim[1] <- min(Ys[1],xlim[1])
		xlim[2] <- max(Ys[n],xlim[2])
		if (!is.null(ab.pw)){
			dimnames(ab.pw)[[2]] <- c('a2.pw','b2.pw')
		}
		if (is.null(ab.sim) && !is.null(kappa)){
			ab.sim <- cbind(pmax(phi-kappa,0),
			                pmin(phi+kappa,1))
		}
		if (!is.null(ab.sim)){
			dimnames(ab.sim)[[2]] <- c('a2.sim','b2.sim')
		}
		if (is.null(xlab)){
			xlab <- expression(italic(x))
		}
		if (is.null(ylab)){
			ylab <- expression(italic(F(x)))
		}
		YYe <- c(xlim[1],rep(Ys,each=2),xlim[2])
		FY <- (0:nY)/nY
		XXe <- c(xlim[1],rep(Xs,each=2),xlim[2])
		# Plot of true F, estimator and confidence bounds:
		par(cex=1.5,mai=c(1.0,1.1,0.01,0.01),
			mgp=c(2.3,1,0))
		plot(YYe,rep(FY,each=2),type='l',
			lwd=3,col='green',
			xlab=xlab,ylab=ylab)
		lines(YYe,rep(FY,each=2),col='darkgreen')
		if (displayFhatM){
			lines(XXe,rep(phi,each=2),col='blue',
				lwd=1+is.null(ab.pw))
		}
		if (!is.null(ab.pw)){
			lines(XXe,rep(ab.pw[,1],each=2),
				lwd=2,col='blue')
			lines(XXe,rep(ab.pw[,2],each=2),
				lwd=2,col='blue')
			lines(XXe,rep(ab.pw[,1],each=2))
			lines(XXe,rep(ab.pw[,2],each=2))
		}
		if (!is.null(ab.sim)){
			lines(XXe,rep(ab.sim[,1],each=2),lwd=2)
			lines(XXe,rep(ab.sim[,2],each=2),lwd=2)
		}		
	}
	FhatTable <- cbind('x'=c(-Inf,Xs),
		'FhatM'=phi,ab.pw,ab.sim)
	return(FhatTable)
}


ErrorAnalysisRSS.finpop <- function(Y,Y.proxy=Y,N,
	yy=unique(sort(Y)),
	phi,ab.pw,kappa=1,
	ab.sim=cbind(pmax(phi-kappa,0),pmin(phi+kappa,1)),
	mcsim=1000,
	display=TRUE,
	alpha=0.05,
	xlab=expression(plain(x)),
	xlim=range(yy))
# In the setup of SimDataRSS.finpop(), this procedure
# simulates mcsim such samples and analyses them with
# respect to
# * bias and root mean squared error of the moment-
#   matching estimator of F,
# * the average widths and coverage probabilities of
#   pointwise confidence intervals, and
# * the average widths and the coverage probability of
#   simultaneous confidence intervals.
# Let n = sum(N), and let
#    -Inf = X_(0) < X_(1) <= ... <= X_(n) < X_(n+1) = Inf
# be the (augmented) order statistics of the RSS sample X.
# For 1 <= i <= n+1 and X_(i-1) <= x < X_(i),
# * the estimator for F(x) equals phi[i],
# * the pointwise confidence interval for F(x) equals
#   [a.pw[i],b.pw[i]],
# * the confidence band for F at x equals
#   [a.sim[i],b.sim[i]].
# In addition the percentage of sample points X[i] with
# incorrect rank R[i] (due to differences between Y and Y.proxy)
# is estimated.
{
	k <- length(N)
	# Initialize output:
	bias <- rep(0,length(yy))
	rmse <- rep(0,length(yy))
	width.pw <- rep(0,length(yy))
	cov.err.pw <- rep(0,length(yy))
	width.sim <- rep(0,length(yy))
	cov.err.sim <- 0
	prop.false.ranks <- 0

	# Preprocess Y:
	nY <- length(Y)
	n <- sum(N)
	if (n*k > nY)
	{
		return('Population too small!')
	}
	
	# Generate rank vector R and initialize observation
	# vector X:
	R <- rep(1:k,times=N)
	X <- rep(NA,n)

	# Compute c.d.f. F at points in yy:
	Fyy <- rep(0,length(yy))
	Ye <- c(sort(Y),Inf)
	iY <- 0
	for (j in 1:length(yy)){
		while (Ye[iY+1] <= yy[j]){
			iY <- iY + 1
		}
		Fyy[j] <- iY/nY
	}

	# MC simulations:
	for (s in 1:mcsim){
		tmp <- SimDataRSS.finpop(Y,Y.proxy,N)
		prop.false.ranks <- prop.false.ranks +
			tmp$prop.false.ranks
		Xe <- c(-Inf,sort(tmp$X),Inf)
		iX <- 1
		tmp.cov.err <- FALSE
		for (j in 1:length(yy)){
			while (Xe[iX+1] <= yy[j]){
				iX <- iX+1
			}
			bias[j] <- bias[j] + phi[iX] - Fyy[j]
			rmse[j] <- rmse[j] + (phi[iX] - Fyy[j])^2
			width.pw[j] <- width.pw[j] +
				ab.pw[iX,2] - ab.pw[iX,1]
			cov.err.pw[j] <- cov.err.pw[j] +
				(ab.pw[iX,1] > Fyy[j] ||
				 ab.pw[iX,2] < Fyy[j])
			width.sim[j] <- width.sim[j] +
				ab.sim[iX,2] - ab.sim[iX,1]
			tmp.cov.err <- tmp.cov.err |
				(ab.sim[iX,1] > Fyy[j] ||
				 ab.sim[iX,2] < Fyy[j])
		}
		cov.err.sim <- cov.err.sim + tmp.cov.err
		if (s %% 100 == 0){
			print(paste(s,'simulations done...'))
		}
	}
	prop.false.ranks <- prop.false.ranks/mcsim
	bias <- bias/mcsim
	rmse <- sqrt(rmse/mcsim)
	width.pw   <- width.pw/mcsim
	cov.err.pw <- cov.err.pw/mcsim
	width.sim   <- width.sim/mcsim
	cov.err.sim <- cov.err.sim/mcsim

	if (display){
		par(cex=1.5,mai=c(1.0,1.1,0.01,0.01),
			mgp=c(2.3,1,0))
		plot(xlim,c(0,0),type='l',col='green',
			xlab=xlab,
			ylim=range(c(bias,rmse)),
			ylab=expression(plain(list(BIAS(x),RMSE(x)))))
		lines(yy,bias,lwd=2,col='red')
		lines(yy,bias)
		lines(yy,rmse,lwd=2,col='blue')
		lines(yy,rmse)
		dev.new()
		par(cex=1.5,mai=c(1.0,1.1,0.01,0.01),
			mgp=c(2.3,1,0))
		plot(xlim,c(0,0),type='l',col='green',
			xlab=xlab,
			ylim=c(0,max(c(width.pw,width.sim))),
			ylab=expression(plain(
				list(AW[pw](x),AW[sim](x)))))
		lines(yy,width.pw,lwd=2,col='blue')
		lines(yy,width.pw)
		lines(yy,width.sim,lwd=2)
		dev.new()
		par(cex=1.5,mai=c(1.0,1.1,0.01,0.01),
			mgp=c(2.3,1,0))
		plot(xlim,c(0,0),type='l',col='green',
			xlab=xlab,
			ylim=c(0,max(cov.err.pw,alpha)),
			ylab=expression(plain(alpha[pw](x))))
		lines(yy,cov.err.pw,lwd=2,col='blue')
		lines(yy,cov.err.pw)
		abline(h=alpha,col='red')
	}
	
	return(list(bias=bias,rmse=rmse,
		width.pw=width.pw,cov.err.pw=cov.err.pw,
		width.sim=width.sim,cov.err.sim=cov.err.sim,
		prop.false.ranks=prop.false.ranks))
}



