############################################################
### Simulating data sets for various JPS/RSS methods     ###
### Lutz Duembgen, Ehsan Zamanzadeh, July 2018           ###
############################################################


### Simulate JPS data with underlying standard uniform
### or standard Gaussian distribution:

SimDataJPS <- function(n,k,distr=c("normal","unif"))
{
	distr <- match.arg(distr,c("normal","unif"))
	R <- ceiling(runif(n,min=0,max=k))
	X <- rbeta(n,R,k+1-R)
	if (distr=="normal"){
		X <- qnorm(X)
	}
	return(list(X=X,R=R,k=k))
}

SimDataJPS.imp <- function(n,k,rho=1)
# Model of Dell and Clutter (1972) with standard
# Gaussian distribution function F.
{
	R <- ceiling(runif(n,min=0,max=k))
	Y <- qnorm(rbeta(n,R,k+1-R))
	X <- rho*Y + rnorm(n,0,sqrt(1 - rho^2))
	return(list(X=X,R=R,k=k))
}


### Simulate RSS data with underlying standard uniform
### or standard Gaussian distribution:

SimDataRSS <- function(N,distr=c("normal","unif"))
{
	distr <- match.arg(distr,c("normal","unif"))
	n <- sum(N)
	k <- length(N)
	R <- rep(1:k,times=N)
	X <- rbeta(n,R,k+1-R)
	if (distr=="normal"){
		X <- qnorm(X)
	}
	return(list(X=X,R=R,k=k))
}

SimDataRSS.imp <- function(N,rho=1)
# Model of Dell and Clutter (1972) with standard
# Gaussian distribution function F.
{
	n <- sum(N)
	k <- length(N)
	R <- rep(1:k,times=N)
	Y <- qnorm(rbeta(n,R,k+1-R))
	X <- rho*Y + rnorm(n,0,sqrt(1 - rho^2))
	return(list(X=X,R=R,k=k))
}


### RSS sampling from a Gaussian population ###
### with ranking errors                     ###

source('Estimation.R')

ErrorAnalysisRSS_2 <- function(N,rho=1,
	yy=qnorm(seq(0.05,0.95,by=0.05)),
	mcsim=1000)
# In the setting of SimDataRSS.imp(), this procedure
# simulates mcsim such samples and analyses the precision
# of the four estimators FhatN, FhatS, FhatM and FhatL
# in terms of bias and root mean squared error.
{
	# Initialize output:
	BIAS <- matrix(0,length(yy),4)
	dimnames(BIAS)[[2]] <- c('N','S','M','L')
	RMSE <- matrix(0,length(yy),4)
	dimnames(RMSE)[[2]] <- c('N','S','M','L')
	
	# Precomputation of FhatN and FhatM:	
	k <- length(N)
	n <- sum(N)
	FhatN <- (0:n)/n
	FhatM <- FhatM_2(N,k)
	
	# Rank vector:
	R <- rep(1:k,times=N)
	
	# Compute c.d.f. F at points in yy:
	Fyy <- pnorm(yy)
	
	# MC simulations:
	for (s in 1:mcsim)
	{
		if (rho < 1)
		{
			# Concomitant observations:
			Y <- qnorm(rbeta(n,R,k+1-R))
			# True observations:
			X <- rho*Y + rnorm(n,0,sqrt(1 - rho^2))
		}
		else
		{
			X <- qnorm(rbeta(n,R,k+1-R))
		}
		tmp1 <- order(X)
		Xs <- X[tmp1]
		Rs <- R[tmp1]
		tmp2 <- Fhat_Data_2(Xs,Rs,N,n,k)
		Ftable <- cbind(FhatN,c(0,tmp2[,1]),FhatM,c(0,tmp2[,2]))
		Xe <- c(-Inf,Xs,Inf)
		iX <- 1
		for (j in 1:length(yy))
		{
			while (Xe[iX+1] <= yy[j])
			{
				iX <- iX+1
			}
			BIAS[j,] <- BIAS[j,] +  Ftable[iX,] - Fyy[j]
			RMSE[j,] <- RMSE[j,] + (Ftable[iX,] - Fyy[j])^2
		}
		if (s %% 100 == 0)
		{
			print(paste(s,'simulations done...'))
		}
	}
	BIAS <- BIAS/mcsim
	RMSE <- sqrt(RMSE/mcsim)
	return(list(BIAS=BIAS,RMSE=RMSE))
}


ErrorAnalysisRSS_3 <- function(N,rho=1,
	yy=qnorm(seq(0.05,0.95,by=0.05)),
	mcsim=1000)
# In the setting of SimDataRSS.imp(), this procedure
# simulates mcsim such samples and analyses the precision
# of the two estimators FhatN and FhatL
# in terms of bias and root mean squared error.
{
	# Initialize output:
	BIAS <- matrix(0,length(yy),2)
	dimnames(BIAS)[[2]] <- c('N','L')
	RMSE <- matrix(0,length(yy),2)
	dimnames(RMSE)[[2]] <- c('N','L')
	
	# Precomputation of FhatN and FhatM:	
	k <- length(N)
	n <- sum(N)
	FhatN <- (0:n)/n

	# Rank vector:
	R <- rep(1:k,times=N)
	
	# Compute c.d.f. F at points in yy:
	Fyy <- pnorm(yy)
	
	# MC simulations:
	for (s in 1:mcsim)
	{
		if (rho < 1)
		{
			# Concomitant observations:
			Y <- qnorm(rbeta(n,R,k+1-R))
			# True observations:
			X <- rho*Y + rnorm(n,0,sqrt(1 - rho^2))
		}
		else
		{
			X <- qnorm(rbeta(n,R,k+1-R))
		}
		tmp1 <- order(X)
		Xs <- X[tmp1]
		Rs <- R[tmp1]
		FhatL <- FhatL(Xs,Rs,N,n,k)
		Ftable <- cbind(FhatN,c(0,FhatL))
		Xe <- c(-Inf,Xs,Inf)
		iX <- 1
		for (j in 1:length(yy))
		{
			while (Xe[iX+1] <= yy[j])
			{
				iX <- iX+1
			}
			BIAS[j,] <- BIAS[j,] +  Ftable[iX,] - Fyy[j]
			RMSE[j,] <- RMSE[j,] + (Ftable[iX,] - Fyy[j])^2
		}
		if (s %% 100 == 0)
		{
			print(paste(s,'simulations done...'))
		}
	}
	if (min(N) == max(N))
	{
		# Utilizing symmetry:
		BIAS <- (BIAS - BIAS[length(yy):1,])/2
		RMSE <- (RMSE + RMSE[length(yy):1,])/2
	}
	BIAS <- BIAS/mcsim
	RMSE <- sqrt(RMSE/mcsim)
	return(list(BIAS=BIAS,RMSE=RMSE))
}
